@@ -18,8 +18,13 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 
 #include <glib.h>
 
@@ -29,6 +34,8 @@
 #include <glib/gi18n.h>
 #endif
 
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 
@@ -425,6 +432,7 @@ main (int argc, gchar **argv)
     GtkWidget *plug_child;
     GladeXML *gxml;
     GError *cli_error = NULL;
+    const gchar  *wm_name;
 
 #ifdef ENABLE_NLS
     xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
@@ -440,6 +448,13 @@ main (int argc, gchar **argv)
         }
     }
 
+    wm_name = gdk_x11_screen_get_window_manager_name (gdk_screen_get_default ());
+    if (G_UNLIKELY (g_ascii_strcasecmp (wm_name, "Xfwm4")))
+    {
+        g_print ("These settings cannot work with your current window manager (%s)\n", wm_name);
+        return 1;
+    }
+
     if (opt_version)
     {
         g_print ("%s\n", PACKAGE_STRING);
