@@ -34,8 +34,9 @@
 #include <glib.h>
 #include <glib-object.h>
 
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-
 #include <glade/glade.h>
 
 #include <libxfce4util/libxfce4util.h>
@@ -404,6 +405,7 @@ xfwm_settings_constructed (GObject *object)
   GtkWidget          *shortcuts_clear_button;
   GtkWidget          *shortcuts_reset_button;
   GtkWidget          *focus_delay_scale;
+  GtkWidget          *focus_raise_delay_scale;
   GtkWidget          *raise_on_click_check;
   GtkWidget          *raise_on_focus_check;
   GtkWidget          *click_to_focus_radio;
@@ -613,6 +615,7 @@ xfwm_settings_constructed (GObject *object)
 
   /* Focus tab widgets */
   focus_delay_scale = glade_xml_get_widget (settings->priv->glade_xml, "focus_delay_scale");
+  focus_raise_delay_scale = glade_xml_get_widget (settings->priv->glade_xml, "focus_raise_delay_scale");
   focus_new_check = glade_xml_get_widget (settings->priv->glade_xml, "focus_new_check");
   raise_on_focus_check = glade_xml_get_widget (settings->priv->glade_xml, "raise_on_focus_check");
   raise_on_click_check = glade_xml_get_widget (settings->priv->glade_xml, "raise_on_click_check");
@@ -621,6 +624,8 @@ xfwm_settings_constructed (GObject *object)
   /* Focus tab */
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/focus_delay", G_TYPE_INT,
                           gtk_range_get_adjustment (GTK_RANGE (focus_delay_scale)), "value");
+  xfconf_g_property_bind (settings->priv->wm_channel, "/general/raise_delay", G_TYPE_INT,
+                          gtk_range_get_adjustment (GTK_RANGE (focus_raise_delay_scale)), "value");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/raise_on_click", G_TYPE_BOOLEAN,
                           raise_on_click_check, "active");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/raise_on_focus", G_TYPE_BOOLEAN,
@@ -926,8 +931,11 @@ main (int    argc,
   GtkWidget    *dialog;
   GtkWidget    *plug;
   GError       *error = NULL;
+  const gchar  *wm_name;
 
+#ifdef ENABLE_NLS
   xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
+#endif
 
   if (G_UNLIKELY (!gtk_init_with_args (&argc, &argv, _("."), opt_entries, PACKAGE, &error)))
     {
@@ -941,6 +949,13 @@ main (int    argc,
       return EXIT_FAILURE;
     }
 
+  wm_name = gdk_x11_screen_get_window_manager_name (gdk_screen_get_default ());
+  if (G_UNLIKELY (g_ascii_strcasecmp (wm_name, "Xfwm4")))
+    {
+      g_print ("These settings cannot work with your current window manager (%s)\n", wm_name);
+      return EXIT_FAILURE;
+    }
+
   if (G_UNLIKELY (opt_version))
     {
       g_print ("%s\n", PACKAGE_STRING);
@@ -1453,6 +1468,7 @@ xfwm_settings_title_alignment_property_changed (XfconfChannel *channel,
   GtkTreeIter   iter;
   GtkWidget    *combo;
   gchar        *alignment;
+  const gchar  *new_value;
 
   g_return_if_fail (XFWM_IS_SETTINGS (settings));
 
@@ -1465,7 +1481,12 @@ xfwm_settings_title_alignment_property_changed (XfconfChannel *channel,
         {
           gtk_tree_model_get (model, &iter, 1, &alignment, -1);
 
-          if (G_UNLIKELY (g_str_equal (alignment, g_value_get_string (value))))
+          if (G_UNLIKELY (G_VALUE_TYPE (value) == G_TYPE_INVALID))
+              new_value = "center";
+          else
+              new_value = g_value_get_string (value);
+
+          if (G_UNLIKELY (g_str_equal (alignment, new_value)))
             {
               g_free (alignment);
               gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
@@ -1915,14 +1936,20 @@ xfwm_settings_validate_shortcut (XfceShortcutDialog  *dialog,
 
   if (G_UNLIKELY (other_shortcut != NULL))
     {
-      response = xfce_shortcut_conflict_dialog (xfce_shortcuts_provider_get_name (settings->priv->provider),
-                                                xfce_shortcuts_provider_get_name (other_provider),
-                                                shortcut,
-                                                xfce_shortcut_dialog_get_action_name (dialog),
-                                                xfwm_settings_shortcut_feature_name (other_shortcut->command),
-                                                FALSE);
+      if (G_LIKELY (!g_str_equal (xfce_shortcut_dialog_get_action (dialog), other_shortcut->command)))
+        {
+          response = xfce_shortcut_conflict_dialog (xfce_shortcuts_provider_get_name (settings->priv->provider),
+                                                    xfce_shortcuts_provider_get_name (other_provider),
+                                                    shortcut,
+                                                    xfce_shortcut_dialog_get_action_name (dialog),
+                                                    xfwm_settings_shortcut_feature_name (other_shortcut->command),
+                                                    FALSE);
 
-      accepted = response == GTK_RESPONSE_ACCEPT;
+          if (G_UNLIKELY (response == GTK_RESPONSE_ACCEPT))
+            xfce_shortcuts_provider_reset_shortcut (other_provider, shortcut);
+          else
+            accepted = FALSE;
+        }
 
       xfce_shortcut_free (other_shortcut);
       g_object_unref (other_provider);
@@ -1965,7 +1992,7 @@ xfwm_settings_shortcut_row_activated (GtkTreeView       *tree_view,
       dialog = xfce_shortcut_dialog_new ("xfwm4", name, feature);
       g_signal_connect (dialog, "validate-shortcut",
                         G_CALLBACK (xfwm_settings_validate_shortcut), settings);
-      response = xfce_shortcut_dialog_run (XFCE_SHORTCUT_DIALOG (dialog));
+      response = xfce_shortcut_dialog_run (XFCE_SHORTCUT_DIALOG (dialog), gtk_widget_get_toplevel (GTK_WIDGET (tree_view)));
 
       if (G_LIKELY (response == GTK_RESPONSE_OK))
         {
