@@ -1,4 +1,4 @@
-/*      $Id: compositor.c 29275 2009-01-18 21:45:33Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -86,7 +86,7 @@
 #define WIN_IS_REDIRECTED(cw)           (cw->redirected)
 
 /* Set TIMEOUT_REPAINT to 0 to disable timeout repaint */
-#define TIMEOUT_REPAINT       20 /* (1/50 sec.) */
+#define TIMEOUT_REPAINT       10 /* msec.) */
 
 typedef struct _CWindow CWindow;
 struct _CWindow
@@ -188,6 +188,29 @@ is_shaped (DisplayInfo *display_info, Window id)
     return FALSE;
 }
 
+static gboolean
+is_fullscreen (CWindow *cw)
+{
+    GdkRectangle rect;
+
+    /* First, check the good old way, the window is larger than the screen size */
+    if (WIN_IS_FULLSCREEN(cw))
+    {
+        return TRUE;
+    }
+
+    /* Next check against the monitors which compose the entire screen */
+    myScreenFindMonitorAtPoint (cw->screen_info,
+                                cw->attr.x + (cw->attr.width + 2 * cw->attr.border_width) / 2,
+                                cw->attr.y + (cw->attr.height + 2 * cw->attr.border_width) / 2,
+                                &rect);
+
+    return ((cw->attr.x == rect.x) &&
+            (cw->attr.y == rect.y) &&
+            (cw->attr.width + 2 * cw->attr.border_width == rect.width) &&
+            (cw->attr.height + 2 * cw->attr.border_width == rect.height));
+}
+
 static gdouble
 gaussian (gdouble r, gdouble x, gdouble y)
 {
@@ -928,8 +951,8 @@ win_extents (CWindow *cw)
 
     if ((screen_info->params->show_popup_shadow &&
               WIN_IS_OVERRIDE(cw) &&
-              !WIN_IS_FULLSCREEN(cw) &&
-              !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw))) ||
+              !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw)) &&
+              !WIN_IS_FULLSCREEN(cw)) ||
           (screen_info->params->show_frame_shadow &&
               !WIN_IS_OVERRIDE(cw) &&
               !WIN_NO_SHADOW(cw) &&
@@ -1464,7 +1487,7 @@ repair_display (DisplayInfo *display_info)
 
     for (screens = display_info->screens; screens; screens = g_slist_next (screens))
     {
-        add_repair ((ScreenInfo *) screens->data);
+        repair_screen ((ScreenInfo *) screens->data);
     }
 }
 #endif
@@ -1745,7 +1768,7 @@ map_win (CWindow *cw)
 
     if (!WIN_IS_REDIRECTED(cw))
     {
-        cw->fulloverlay = WIN_IS_FULLSCREEN(cw);
+        cw->fulloverlay = is_fullscreen(cw);
         if (cw->fulloverlay)
         {
             /*
@@ -1771,23 +1794,14 @@ map_win (CWindow *cw)
     cw->viewable = TRUE;
     cw->damaged = FALSE;
 
-    if (!screen_info->params->unredirect_overlays)
-    {
-        TRACE ("Not unredirecting wins_unredirected");
-        return;
-    }
-
     /* Check for new windows to un-redirect. */
-    if ((WIN_IS_FULLSCREEN(cw) || (screen_info->wins_unredirected > 0)) && WIN_HAS_DAMAGE(cw) && WIN_IS_OVERRIDE(cw) &&
-        WIN_IS_NATIVE_OPAQUE(cw) && WIN_IS_REDIRECTED(cw) && !WIN_IS_SHAPED(cw))
+    if (WIN_HAS_DAMAGE(cw) && WIN_IS_OVERRIDE(cw) &&
+        WIN_IS_NATIVE_OPAQUE(cw) && WIN_IS_REDIRECTED(cw) && !WIN_IS_SHAPED(cw)
+        && ((screen_info->wins_unredirected > 0) || is_fullscreen(cw)))
     {
-        CWindow *top;
-        GList *index;
-
-        index = screen_info->cwindows;
-        top = (CWindow *) index->data;
-
-        if (cw == top)
+        /* Make those opaque, we don't want them to be transparent */
+        cw->opacity = NET_WM_OPAQUE;
+        if (screen_info->params->unredirect_overlays)
         {
             TRACE ("Unredirecting toplevel window 0x%lx", cw->id);
             unredirect_win (cw);
